<?php
/**
 * File containing the AddFieldTypePass class.
 *
 * @copyright Copyright (C) 1999-2014 eZ Systems AS. All rights reserved.
 * @license http://ez.no/licenses/gnu_gpl GNU General Public License v2.0
 * @version 
 */

namespace eZ\Bundle\EzPublishCoreBundle\DependencyInjection\Compiler;

use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\Reference;

/**
 * This compiler pass will register eZ Publish field types.
 */
class AddFieldTypePass implements CompilerPassInterface
{
    /**
     * @param \Symfony\Component\DependencyInjection\ContainerBuilder $container
     *
     * @throws \LogicException
     */
    public function process( ContainerBuilder $container )
    {
        if ( !$container->hasDefinition( 'ezpublish.api.repository.factory' ) )
            return;

        $repositoryFactoryDef = $container->getDefinition( 'ezpublish.api.repository.factory' );

        // Field types.
        // Alias attribute is the field type string.
        foreach ( $container->findTaggedServiceIds( 'ezpublish.fieldType' ) as $id => $attributes )
        {
            foreach ( $attributes as $attribute )
            {
                if ( !isset( $attribute['alias'] ) )
                    throw new \LogicException( 'ezpublish.fieldType service tag needs an "alias" attribute to identify the field type. None given.' );

                $repositoryFactoryDef->addMethodCall(
                    'registerFieldType',
                    array(
                        // Only pass the service Id since field types will be lazy loaded via the service container
                        $id,
                        $attribute['alias']
                    )
                );
            }
        }

        // Parameter providers for field types.
        $parameterProviderRegistryDef = $container->getDefinition( 'ezpublish.fieldType.parameterProviderRegistry' );
        foreach ( $container->findTaggedServiceIds( 'ezpublish.fieldType.parameterProvider' ) as $id => $attributes )
        {
            foreach ( $attributes as $attribute )
            {
                if ( !isset( $attribute['alias'] ) )
                    throw new \LogicException( 'ezpublish.fieldType.parameterProvider service tag needs an "alias" attribute to identify the field type. None given.' );

                $parameterProviderRegistryDef->addMethodCall(
                    'setParameterProvider',
                    array(
                        // Only pass the service Id since field types will be lazy loaded via the service container
                        new Reference( $id ),
                        $attribute['alias']
                    )
                );
            }
        }

        // Gateways for external storage handlers.
        // Alias attribute is the corresponding field type string.
        $externalStorageGateways = array();
        // Referencing the services by alias (field type string)
        foreach ( $container->findTaggedServiceIds( 'ezpublish.fieldType.externalStorageHandler.gateway' ) as $id => $attributes )
        {
            foreach ( $attributes as $attribute )
            {
                if ( !isset( $attribute['alias'] ) )
                    throw new \LogicException( 'ezpublish.fieldType.externalStorageHandler.gateway service tag needs an "alias" attribute to identify the field type. None given.' );

                if ( !isset( $attribute['identifier'] ) )
                    throw new \LogicException( 'ezpublish.fieldType.externalStorageHandler.gateway service tag needs an "identifier" attribute to identify the gateway. None given.' );

                $externalStorageGateways[$attribute['alias']] = array(
                    'id'            => $id,
                    'identifier'    => $attribute['identifier']
                );
            }
        }

        // External storage handlers for field types that need them.
        // Alias attribute is the field type string.
        foreach ( $container->findTaggedServiceIds( 'ezpublish.fieldType.externalStorageHandler' ) as $id => $attributes )
        {
            foreach ( $attributes as $attribute )
            {
                if ( !isset( $attribute['alias'] ) )
                    throw new \LogicException( 'ezpublish.fieldType.externalStorageHandler service tag needs an "alias" attribute to identify the field type. None given.' );

                // If the storage handler is gateway based, then we need to add a corresponding gateway to it.
                // Will throw a LogicException if no gateway is defined for this field type.
                $storageHandlerDef = $container->findDefinition( $id );
                $storageHandlerClass = $storageHandlerDef->getClass();
                if ( preg_match( '/^%([^%\s]+)%$/', $storageHandlerClass, $match ) )
                    $storageHandlerClass = $container->getParameter( $match[1] );

                if (
                    is_subclass_of(
                        $storageHandlerClass,
                        'eZ\\Publish\\Core\\FieldType\\GatewayBasedStorage'
                    )
                )
                {
                    if ( !isset( $externalStorageGateways[$attribute['alias']] ) )
                        throw new \LogicException(
                            "External storage handler '$id' for field type {$attribute['alias']} needs a storage gateway but none was given.
                        Consider defining a storage gateway as a service for this field type and add the 'ezpublish.fieldType.externalStorageHandler.gateway tag'"
                        );

                    $storageHandlerDef->addMethodCall(
                        'addGateway',
                        array(
                            $externalStorageGateways[$attribute['alias']]['identifier'],
                            new Reference( $externalStorageGateways[$attribute['alias']]['id'] )
                        )
                    );
                }

                $repositoryFactoryDef->addMethodCall(
                    'registerExternalStorageHandler',
                    array(
                        // Only pass the service Id since field types will be lazy loaded via the service container
                        $id,
                        $attribute['alias']
                    )
                );
            }
        }
    }
}
